@@ -79,7 +79,7 @@ widget_get_id_string (GtkWidget *widget)
         g_return_val_if_fail (widget != NULL, NULL);
 
         id = g_strdup_printf ("0x%X",
-                              (guint32)GDK_WINDOW_XID (widget->window));
+                              (guint32)GDK_WINDOW_XID (gtk_widget_get_window (widget)));
         return id;
 }
 
@@ -189,6 +189,9 @@ gs_job_set_command  (GSJob      *job,
 {
         g_return_val_if_fail (GS_IS_JOB (job), FALSE);
 
+        gs_debug ("Setting command for job: '%s'",
+                  command != NULL ? command : "NULL");
+
         g_free (job->priv->command);
         job->priv->command = g_strdup (command);
 
@@ -250,7 +253,8 @@ get_env_vars (GtkWidget *widget)
                 "XAUTHORITY",
                 "XAUTHLOCALHOSTNAME",
                 "LANG",
-                "LANGUAGE"
+                "LANGUAGE",
+                "DBUS_SESSION_BUS_ADDRESS"
         };
 
         env = g_ptr_array_new ();
@@ -314,18 +318,17 @@ spawn_on_widget (GtkWidget  *widget,
         env = get_env_vars (widget);
 
         error = NULL;
-        result = gdk_spawn_on_screen_with_pipes (gtk_widget_get_screen (widget),
-                                                 NULL,
-                                                 argv,
-                                                 (char **)env->pdata,
-                                                 G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD,
-                                                 NULL,
-                                                 NULL,
-                                                 &child_pid,
-                                                 NULL,
-                                                 NULL,
-                                                 &standard_error,
-                                                 &error);
+        result = g_spawn_async_with_pipes (NULL,
+                                           argv,
+                                           (char **)env->pdata,
+                                           G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD,
+                                           NULL,
+                                           NULL,
+                                           &child_pid,
+                                           NULL,
+                                           NULL,
+                                           &standard_error,
+                                           &error);
         for (i = 0; i < env->len; i++) {
                 g_free (g_ptr_array_index (env, i));
         }
@@ -382,12 +385,16 @@ command_watch (GIOChannel   *source,
 
                 status = g_io_channel_read_line (source, &str, NULL, NULL, &error);
 
-		if (status == G_IO_STATUS_NORMAL) {
+                if (status == G_IO_STATUS_NORMAL) {
                         gs_debug ("command output: %s", str);
 
-		} else if (status == G_IO_STATUS_EOF) {
+                } else if (status == G_IO_STATUS_EOF) {
                         done = TRUE;
-		}
+
+                } else if (error != NULL) {
+                        gs_debug ("command error: %s", error->message);
+                        g_error_free (error);
+                }
 
                 g_free (str);
         } else if (condition & G_IO_HUP) {
@@ -439,6 +446,7 @@ gs_job_start (GSJob *job)
         if (job->priv->command == NULL) {
                 /* no warning here because a NULL command is interpreted
                    as a no-op job */
+                gs_debug ("No command set for job.");
                 return FALSE;
         }
 
